@@ -27,7 +27,7 @@ SOFTWARE.
 """
 
 
-__version__ = '0.5.5'
+__version__ = '0.6.0'
 __title__ = 'ssqueezepy'
 __author__ = 'OverLordGoldDragon'
 __license__ = __doc__
@@ -39,19 +39,27 @@ from . import _cwt
 from . import _stft
 from . import _ssq_cwt
 from . import _ssq_stft
+from . import _gmw
+from . import _test_signals
 from . import wavelets
+from . import ridge_extraction
 from . import utils
 from . import toolkit
 from . import visuals
 from . import algos
+from . import configs
 from . import experimental
 
+
 from .ssqueezing import *
-from ._ssq_cwt import *
-from ._ssq_stft import *
 from ._cwt import *
 from ._stft import *
+from ._ssq_cwt import *
+from ._ssq_stft import *
+from ._gmw import *
+from ._test_signals import *
 from .wavelets import *
+from .ridge_extraction import *
 
 
 def wavs():
