@@ -1,14 +1,15 @@
 # -*- coding: utf-8 -*-
 import numpy as np
 from numpy.fft import fft, ifft, ifftshift
-from .utils import WARN, p2up, adm_cwt, adm_ssq, _process_fs_and_t
-from .utils import padsignal, process_scales
+from .utils import WARN, adm_cwt, adm_ssq, _process_fs_and_t
+from .utils import padsignal, process_scales, logscale_transition_idx
 from .algos import replace_at_inf_or_nan
 from .wavelets import Wavelet
 
 
-def cwt(x, wavelet='morlet', scales='log', fs=None, t=None, nv=32, l1_norm=True,
-        derivative=False, padtype='reflect', rpadded=False, vectorized=True):
+def cwt(x, wavelet='gmw', scales='log-piecewise', fs=None, t=None, nv=32,
+        l1_norm=True, derivative=False, padtype='reflect', rpadded=False,
+        vectorized=True, order=0, average=None):
     """Continuous Wavelet Transform, discretized, as described in
     Sec. 4.3.3 of [1] and Sec. IIIA of [2]. Uses a form of discretized
     convolution theorem via wavelets in the Fourier domain and FFT of input.
@@ -19,31 +20,37 @@ def cwt(x, wavelet='morlet', scales='log', fs=None, t=None, nv=32, l1_norm=True,
 
         wavelet: str / tuple[str, dict] / `wavelets.Wavelet`
             Wavelet sampled in Fourier frequency domain.
-                - str: name of builtin wavelet. `ssqueezepy.wavs()`
-                - tuple[str, dict]: name of builtin wavelet and its configs.
+                - str: name of builtin wavelet. See `ssqueezepy.wavs()`
+                  or `Wavelet.SUPPORTED`.
+                - tuple: name of builtin wavelet and its configs.
                   E.g. `('morlet', {'mu': 5})`.
                 - `wavelets.Wavelet` instance. Can use for custom wavelet.
-                  See help(wavelets.Wavelet).
+                  See `help(wavelets.Wavelet)`.
 
-        scales: str['log', 'linear', 'log:maximal', ...] / np.ndarray
+        scales: str['log', 'log-piecewise', 'linear', 'log:maximal', ...]
+                / np.ndarray
             CWT scales.
                 - 'log': exponentially distributed scales, as pow of 2:
                          `[2^(1/nv), 2^(2/nv), ...]`
+                - 'log-piecewise': 'log' except very high `scales` are downsampled
+                  to prevent redundancy. This is recommended. See
+                  https://github.com/OverLordGoldDragon/ssqueezepy/issues/
+                  29#issuecomment-776792726
                 - 'linear': linearly distributed scales.
-                  !!! EXPERIMENTAL; default scheme for len(x)>2048 performs
-                  poorly (and there may not be a good non-piecewise scheme).
+                  !!! this scheme is not recommended; use with caution
 
-            str assumes default `preset='maximal'`, which can be changed via
-            e.g. 'log:maximal', 'linear:minimal'. See `preset` in
-            help(utils.cwt_scalebounds).
+            str assumes default `preset` of `'minimal'` for low scales and
+            `'maximal'` for high, which can be changed via e.g. 'log:maximal'.
+            See `preset` in `help(utils.cwt_scalebounds)`.
 
         nv: int
-            Number of voices. Suggested >= 32.
+            Number of voices (wavelets per octave). Suggested >= 16.
 
         fs: float / None
-            Sampling frequency of `x`. Defaults to 1, which makes ssq
-            frequencies range from 1/dT to 0.5*fs, i.e. as fraction of reference
-            sampling rate up to Nyquist limit; dT = total duration (N/fs).
+            Sampling frequency of `x`. Defaults to 1, which for
+            `maprange='maximal'` makes ssq frequencies range from 1/dT to 0.5*fs,
+            i.e. as fraction of reference sampling rate up to Nyquist limit;
+            dT = total duration (N/fs).
             Used to compute `dt`, which is only used if `derivative=True`.
             Overridden by `t`, if provided.
             Relevant on `t` and `dT`: https://dsp.stackexchange.com/a/71580/50076
@@ -63,8 +70,9 @@ def cwt(x, wavelet='morlet', scales='log', fs=None, t=None, nv=32, l1_norm=True,
         derivative: bool (default False)
             Whether to compute and return `dWx`. Requires `fs` or `t`.
 
-        padtype: str
+        padtype: str / None
             Pad scheme to apply on input. See `help(utils.padsignal)`.
+            `None` -> no padding.
 
         rpadded: bool (default False)
              Whether to return padded Wx and dWx.
@@ -75,6 +83,13 @@ def cwt(x, wavelet='morlet', scales='log', fs=None, t=None, nv=32, l1_norm=True,
             Whether to compute quantities for all scales at once, which is
             faster but uses more memory.
 
+        order: int (default 0) / tuple[int] / range
+            > 0 computes `cwt` with higher-order GMWs. If tuple, computes
+            `cwt` at each specified order. See `help(_cwt.cwt_higher_order)`.
+
+        average: bool / None
+            Only used for tuple `order`; see `help(_cwt.cwt_higher_order)`.
+
     # Returns:
         Wx: [na x n] np.ndarray (na = number of scales; n = len(x))
             CWT of `x`. (rows=scales, cols=timeshifts)
@@ -123,10 +138,10 @@ def cwt(x, wavelet='morlet', scales='log', fs=None, t=None, nv=32, l1_norm=True,
         if derivative:
             dWx = Wx.copy()
 
-        for i, a in enumerate(scales):
+        for i, scale in enumerate(scales):
             # sample FT of wavelet at scale `a`
             # * pn = freq-domain spectral reversal to center time-domain wavelet
-            psih = wavelet(scale=a, nohalf=False) * pn
+            psih = wavelet(scale=scale, nohalf=False) * pn
             Wx[i] = ifftshift(ifft(psih * xh))
 
             if derivative:
@@ -135,35 +150,57 @@ def cwt(x, wavelet='morlet', scales='log', fs=None, t=None, nv=32, l1_norm=True,
         return (Wx, dWx) if derivative else (Wx, None)
 
     def _process_args(x, scales, nv, fs, t):
+        if not isinstance(x, np.ndarray):
+            raise TypeError("`x` must be a numpy array (got %s)" % type(x))
+        elif x.ndim != 1:
+            raise ValueError("`x` must be 1D (got x.ndim == %s)" % x.ndim)
+
         if np.isnan(x.max()) or np.isinf(x.max()) or np.isinf(x.min()):
             WARN("found NaN or inf values in `x`; will zero")
             replace_at_inf_or_nan(x, replacement=0.)
+
         if isinstance(scales, np.ndarray):
             nv = None
-        dt, *_ = _process_fs_and_t(fs, t, N=len(x))
-        return nv, dt  # == 1 / fs
+        N = len(x)
+        dt, *_ = _process_fs_and_t(fs, t, N=N)
+        return N, nv, dt
 
-    nv, dt = _process_args(x, scales, nv, fs, t)
+    if isinstance(order, (tuple, list, range)) or order > 0:
+        kw = dict(wavelet=wavelet, scales=scales, fs=fs, t=t, nv=nv,
+                  l1_norm=l1_norm, derivative=derivative, padtype=padtype,
+                  rpadded=rpadded, vectorized=vectorized)
+        return cwt_higher_order(x, order=order, average=average, **kw)
 
-    xp, nup, n1, _ = padsignal(x, padtype, get_params=True)
+    N, nv, dt = _process_args(x, scales, nv, fs, t)
 
+    if padtype is not None:
+        xp, _, n1, _ = padsignal(x, padtype, get_params=True)
+    else:
+        xp = x
+
+    # zero-mean `xp`, take to freq-domain
     xp -= xp.mean()
-    xh = fft(xp)
-    wavelet = Wavelet._init_if_not_isinstance(wavelet)
-    scales = process_scales(scales, len(x), wavelet, nv=nv)
-    pn = (-1)**np.arange(nup)
+    xh = fft(xp, axis=-1)
 
-    N_orig = wavelet.N
-    wavelet.N = nup
+    # validate `wavelet`, process `scales`, define `pn` for later
+    wavelet = _process_gmw_wavelet(wavelet, l1_norm)
+    wavelet = Wavelet._init_if_not_isinstance(wavelet)
+    scales = process_scales(scales, N, wavelet, nv=nv)
+    pn = (-1)**np.arange(len(xp))
+
+    # temporarily adjust `wavlet.N`, take CWT
+    wavelet_N_orig = wavelet.N
+    wavelet.N = len(xp)
     Wx, dWx = (_vectorized(xh, scales, wavelet, pn, derivative) if vectorized else
                _for_loop(  xh, scales, wavelet, pn, derivative))
-    wavelet.N = N_orig
+    wavelet.N = wavelet_N_orig  # restore
 
-    if not rpadded:
+    # handle unpadding, normalization
+    if not rpadded and padtype is not None:
         # shorten to pre-padded size
-        Wx = Wx[:, n1:n1 + len(x)]
+        Wx = Wx[:, n1:n1 + N]
         if derivative:
-            dWx = dWx[:, n1:n1 + len(x)]
+            dWx = dWx[:, n1:n1 + N]
     if not l1_norm:
         # normalize energy per L2 wavelet norm, else already L1-normalized
         Wx *= np.sqrt(scales)
@@ -174,9 +211,9 @@ def cwt(x, wavelet='morlet', scales='log', fs=None, t=None, nv=32, l1_norm=True,
             (Wx, scales))
 
 
-def icwt(Wx, wavelet, scales='log', nv=None, one_int=True, x_len=None, x_mean=0,
-         padtype='zero', rpadded=False, l1_norm=True):
-    """The inverse continuous wavelet transform of Wx, via double or
+def icwt(Wx, wavelet='gmw', scales='log-piecewise', nv=None, one_int=True,
+         x_len=None, x_mean=0, padtype='zero', rpadded=False, l1_norm=True):
+    """The inverse Continuous Wavelet Transform of `Wx`, via double or
     single integral.
 
     # Arguments:
@@ -210,8 +247,8 @@ def icwt(Wx, wavelet, scales='log', nv=None, one_int=True, x_len=None, x_mean=0,
             infinite scale component). Default 0.
 
         padtype: str
-            Pad scheme to apply on input. See `help(utils.padsignal)`.
-            !!! currently uses only 'zero'
+            Pad scheme to apply on input, in case of `one_int=False`.
+            See `help(utils.padsignal)`.
 
         rpadded: bool (default False)
             True if Wx is padded (e.g. if used `cwt(, rpadded=True)`).
@@ -252,11 +289,26 @@ def icwt(Wx, wavelet, scales='log', nv=None, one_int=True, x_len=None, x_mean=0,
     if not isinstance(scales, np.ndarray) and nv is None:
         nv = 32  # must match forward's; default to `cwt`'s
 
+    wavelet = _process_gmw_wavelet(wavelet, l1_norm)
     wavelet = Wavelet._init_if_not_isinstance(wavelet)
+    # will override `nv` to match `scales`'s
     scales, scaletype, _, nv = process_scales(scales, x_len, wavelet, nv=nv,
                                               get_params=True)
     assert (len(scales) == na), "%s != %s" % (len(scales), na)
 
+    #### Handle piecewise scales case ########################################
+    # `nv` must be left unspecified so it's inferred automatically from `scales`
+    # in `process_scales` for each piecewise case
+    if scaletype == 'log-piecewise':
+        kw = dict(wavelet=wavelet, one_int=one_int, x_len=x_len, x_mean=x_mean,
+                  padtype=padtype, rpadded=rpadded, l1_norm=l1_norm)
+
+        idx = logscale_transition_idx(scales)
+        x  = icwt(Wx[:idx], scales=scales[:idx], **kw)
+        x += icwt(Wx[idx:], scales=scales[idx:], **kw)
+        return x
+    ##########################################################################
+
     #### Invert ##############################################################
     if one_int:
         x = _icwt_1int(Wx, scales, scaletype, l1_norm)
@@ -281,20 +333,19 @@ def icwt(Wx, wavelet, scales='log', nv=None, one_int=True, x_len=None, x_mean=0,
 def _icwt_2int(Wx, scales, scaletype, l1_norm, wavelet, x_len,
                padtype='zero', rpadded=False):
     """Double-integral iCWT; works with any(?) wavelet."""
-    nup, n1, n2 = p2up(x_len)
-    # add CWT padding if it doesn't exist  # TODO symmetric & other?
+    # add CWT padding if it doesn't exist
     if not rpadded:
-        Wx = np.pad(Wx, [[0, 0], [n1, n2]])  # pad time axis, left=n1, right=n2
+        Wx, n_up, n1, _ = padsignal(Wx, padtype=padtype, get_params=True)
 
     # see help(cwt) on `norm` and `pn`
     norm = _icwt_norm(scaletype, l1_norm)
-    pn = (-1)**np.arange(nup)
-    x = np.zeros(nup)
+    pn = (-1)**np.arange(n_up)
+    x = np.zeros(n_up)
 
-    for a, Wxa in zip(scales, Wx):  # TODO vectorize?
-        psih = wavelet(scale=a, N=nup) * pn
-        xa = ifftshift(ifft(fft(Wxa) * psih))  # convolution theorem
-        x += xa.real / norm(a)
+    for scale, Wx_scale in zip(scales, Wx):  # TODO vectorize?
+        psih = wavelet(scale=scale, N=n_up) * pn
+        xa = ifftshift(ifft(fft(Wx_scale) * psih))  # convolution theorem
+        x += xa.real / norm(scale)
 
     x = x[n1:n1 + x_len]  # keep the unpadded part
     return x
@@ -308,11 +359,123 @@ def _icwt_1int(Wx, scales, scaletype, l1_norm):
 
 def _icwt_norm(scaletype, l1_norm):
     if l1_norm:
-        norm = ((lambda a: 1) if scaletype == 'log' else
-                (lambda a: a))
+        norm = ((lambda scale: 1) if scaletype == 'log' else
+                (lambda scale: scale))
     else:
         if scaletype == 'log':
-            norm = lambda a: a**.5
+            norm = lambda scale: scale**.5
         elif scaletype == 'linear':
-            norm = lambda a: a**1.5
+            norm = lambda scale: scale**1.5
     return norm
+
+
+def _process_gmw_wavelet(wavelet, l1_norm):
+    """Ensure `norm` for GMW is consistent with `l1_norm`."""
+    norm = 'bandpass' if l1_norm else 'energy'
+
+    if isinstance(wavelet, str) and wavelet.lower()[:3] == 'gmw':
+        wavelet = ('gmw', {'norm': norm})
+
+    elif isinstance(wavelet, tuple) and wavelet[0].lower()[:3] == 'gmw':
+        wavelet, wavopts = wavelet
+        wavopts['norm'] = wavopts.get('norm', norm)
+        wavelet = (wavelet, wavopts)
+
+    elif isinstance(wavelet, Wavelet):
+        if wavelet.name == 'GMW L2' and l1_norm:
+            raise ValueError("using GMW L2 wavelet with `l1_norm=True`")
+        elif wavelet.name == 'GMW L1' and not l1_norm:
+            raise ValueError("using GMW L1 wavelet with `l1_norm=False`")
+    return wavelet
+
+
+def cwt_higher_order(x, wavelet='gmw', order=1, average=None, **kw):
+    """Compute `cwt` with GMW wavelets of order 0 to `order`. See `help(cwt)`.
+
+    Yields lower variance and more noise robust representation. See VI in ref[1].
+
+    # Arguments:
+        x: np.ndarray
+            Input, 1D.
+
+        wavelet: str / wavelets.Wavelet
+            CWT wavelet.
+
+        order: int / tuple[int] / range
+            Order of GMW to use for CWT. If tuple, will compute for each
+            order specified in tuple, subject to `average`.
+
+        average: bool (default True if `order` is tuple)
+            If True, will take arithmetic mean of resulting `Wx` (and `dWx`
+            if `derivative=True`), else return as list. Note for phase transform,
+            one should compute derivative of averaged `Wx` rather than take
+            average of individual `dWx`s.
+            Ignored with non-tuple `order.
+
+        kw: dict / kwargs
+            Arguments to `cwt`.
+            If `scales` is string, will reuse zeroth-order's; zeroth order
+            isn't included in `order`, will set from wavelet at `order=0`.
+
+    # References
+        [1] Generalized Morse Wavelets. S. C. Olhede, A. T. Walden. 2002.
+        https://spiral.imperial.ac.uk/bitstream/10044/1/1150/1/
+        OlhedeWaldenGenMorse.pdf
+    """
+    def _process_wavelet(wavelet, order):
+        wavelet = Wavelet._init_if_not_isinstance(wavelet)
+        if not wavelet.name.lower().startswith('gmw'):
+            raise ValueError("`wavelet` must be GMW for higher-order transforms "
+                             "(got %s)" % wavelet.name)
+
+        wavopts = wavelet.config.copy()
+        wavopts.pop('order')
+        wavelets = [Wavelet(('gmw', dict(order=k, **wavopts))) for k in order]
+        return wavelets, wavopts
+
+    def _process_args(wavelet, order, average, kw):
+        if isinstance(order, (list, range)):
+            order = tuple(order)
+        if not isinstance(order, (list, tuple)):
+            order = [order]
+        if len(order) == 1 and average:
+            WARN("`average` ignored with single `order`")
+            average = False
+
+        wavelets, wavopts = _process_wavelet(wavelet, order)
+
+        scales = kw.get('scales', 'log-piecewise')
+        if isinstance(scales, str):
+            wav = Wavelet(('gmw', dict(order=0, **wavopts)))
+            scales = process_scales(scales, len(x), wavelet=wav,
+                                    nv=kw.get('nv', 32))
+        kw['scales'] = scales
+
+        return wavelets, order, average
+
+    wavelets, order, average = _process_args(wavelet, order, average, kw)
+
+    Wx_all = []
+    derivative = kw.get('derivative', False)
+    if derivative:
+        dWx_all = []
+
+    # take the CWTs
+    for k in range(len(order)):
+        out = cwt(x, wavelets[k], order=0, **kw)
+        Wx_all.append(out[0])
+        if derivative:
+            dWx_all.append(out[-1])
+
+    # handle averaging; strip `Wx_all` of list container if only one array
+    if average or (average is None and isinstance(order, tuple)):
+        Wx_all = np.mean(np.vstack([Wx_all]), axis=0)
+        if derivative:
+            dWx_all = np.mean(np.vstack([dWx_all]), axis=0)
+    elif len(Wx_all) == 1:
+        Wx_all = Wx_all[0]
+        if derivative:
+            dWx_all = dWx_all[0]
+
+    return ((Wx_all, kw['scales'], dWx_all) if derivative else
+            (Wx_all, kw['scales']))
